package cc_petitions.admin.survey.web;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import cc_petitions.admin.service.AdminVO;
import cc_petitions.survey.service.SurveyService;
import cc_petitions.survey.service.SurveyVO;
import egovframework.com.cmm.EgovMessageSource;
import egovframework.com.cop.bbs.service.BoardVO;
import egovframework.rte.fdl.property.EgovPropertyService;
import egovframework.rte.ptl.mvc.tags.ui.pagination.PaginationInfo;

@Controller
@RequestMapping(value = "/admin/survey")
public class AdminSurveyController{
	
	@Resource(name = "surveyService")
	private SurveyService surveyService;
	
	@Resource(name = "propertiesService")
	protected EgovPropertyService propertyService;
	
	@Resource(name = "egovMessageSource")
    protected EgovMessageSource egovMessageSource;

	/**
     * XSS 방지 처리.
     * 
     * @param data
     * @return
     */
    protected String unscript(String data) {
        if (data == null || data.trim().equals("")) {
            return "";
        }
        
        String ret = data;
        
        ret = ret.replaceAll("<(S|s)(C|c)(R|r)(I|i)(P|p)(T|t)", "&lt;script");
        ret = ret.replaceAll("</(S|s)(C|c)(R|r)(I|i)(P|p)(T|t)", "&lt;/script");
        
        ret = ret.replaceAll("<(O|o)(B|b)(J|j)(E|e)(C|c)(T|t)", "&lt;object");
        ret = ret.replaceAll("</(O|o)(B|b)(J|j)(E|e)(C|c)(T|t)", "&lt;/object");
        
        ret = ret.replaceAll("<(A|a)(P|p)(P|p)(L|l)(E|e)(T|t)", "&lt;applet");
        ret = ret.replaceAll("</(A|a)(P|p)(P|p)(L|l)(E|e)(T|t)", "&lt;/applet");
        
        ret = ret.replaceAll("<(E|e)(M|m)(B|b)(E|e)(D|d)", "&lt;embed");
        ret = ret.replaceAll("</(E|e)(M|m)(B|b)(E|e)(D|d)", "&lt;embed");
        
        ret = ret.replaceAll("<(F|f)(O|o)(R|r)(M|m)", "&lt;form");
        ret = ret.replaceAll("</(F|f)(O|o)(R|r)(M|m)", "&lt;form");

        return ret;
    }
    
	@RequestMapping(value = "survey.do")
	public String selectSurveyList(HttpServletRequest request, SurveyVO surveyVO, ModelMap model) throws Exception {
		AdminVO user = (AdminVO) request.getSession().getAttribute("adminVO");
		
		if (user == null) {
			return "redirect:/admin/login.do";
		}else{
			PaginationInfo paginationInfo = new PaginationInfo();
			
			paginationInfo.setCurrentPageNo(surveyVO.getPageIndex());
			paginationInfo.setRecordCountPerPage(surveyVO.getPageUnit());
			paginationInfo.setPageSize(surveyVO.getPageSize());
			
			surveyVO.setFirstIndex(paginationInfo.getFirstRecordIndex());
			surveyVO.setLastIndex(paginationInfo.getLastRecordIndex());
			surveyVO.setRecordCountPerPage(paginationInfo.getRecordCountPerPage());
			
			Map<String, Object> map = surveyService.selectSurveyArticles(surveyVO);// 2011.09.07
			int totCnt = Integer.parseInt((String) map.get("resultCnt"));
			
			paginationInfo.setTotalRecordCount(totCnt);
			
			model.addAttribute("resultList", map.get("resultList"));
			model.addAttribute("resultCnt", map.get("resultCnt"));
			model.addAttribute("paginationInfo", paginationInfo);
			model.addAttribute("pageIndex", surveyVO.getPageIndex());
			model.addAttribute("curAdmin", user);
			
			return "/admin/survey/EgovSurvey";
		}
	}
	
	@RequestMapping(value = "AddSurvey.do", method = RequestMethod.GET)
	public String AddSurvey(HttpServletRequest request, SurveyVO surveyVO, ModelMap model) throws Exception {
		AdminVO user = (AdminVO) request.getSession().getAttribute("adminVO");
		if (user == null) {
			return "redirect:/admin/login.do";
		}else{
			model.addAttribute("surveyFlag", surveyVO.getSurvey_flag());
			model.addAttribute("curAdmin", user);
			
			return "/admin/survey/EgovSurveyWrite";
		}
	}
	
	@RequestMapping(value = "AddSurveyOk.do")
	public String AddSurveyOk(HttpServletRequest request, SurveyVO surveyVO, ModelMap model) throws Exception {
		AdminVO user = (AdminVO) request.getSession().getAttribute("adminVO");
		if (user == null) {
			return "redirect:/admin/login.do";
		}else{
			//surveyVO.setQue_cn(unscript(surveyVO.getQue_cn()));	// XSS 방지
			surveyService.insertSurvey(surveyVO);
			// 등록 후 리스트 화면이동
			return "redirect:/admin/survey/survey.do";
		}
	}
	
	@RequestMapping(value = "ViewSurvey.do", method = RequestMethod.GET)
	public String ViewSurvey(HttpServletRequest request, SurveyVO surveyVO, ModelMap model) throws Exception {
		AdminVO user = (AdminVO) request.getSession().getAttribute("adminVO");
		if (user == null) {
			return "redirect:/admin/login.do";
		}else{
			SurveyVO survey = surveyService.getSurvey(surveyVO.getQue_id());
			String pageIndex = request.getParameter("pageIndex")==null?"1":request.getParameter("pageIndex");
			
			model.addAttribute("survey", survey);
			model.addAttribute("pageIndex", pageIndex);
			model.addAttribute("curAdmin", user);
			
			return "/admin/survey/EgovSurveyView";
		}
	}
	
	@RequestMapping(value = "UpdateSurvey.do", method = RequestMethod.GET)
	public String UpdateSurvey(HttpServletRequest request, SurveyVO surveyVO, ModelMap model) throws Exception {
		AdminVO user = (AdminVO) request.getSession().getAttribute("adminVO");
		if (user == null) {
			return "redirect:/admin/login.do";
		}else{
			// 설문조사 조회
			SurveyVO survey = surveyService.getSurvey(surveyVO.getQue_id());
			model.addAttribute("survey", survey);		
			model.addAttribute("curAdmin", user);
			
			return "/admin/survey/EgovSurveyUpdate";
		}
	}
	
	@RequestMapping(value = "UpdateSurveyOk.do", method = RequestMethod.POST)
	public String UpdateSurveyOk(HttpServletRequest request, SurveyVO surveyVO, ModelMap model) throws Exception {
		AdminVO user = (AdminVO) request.getSession().getAttribute("adminVO");
		if (user == null) {
			return "redirect:/admin/login.do";
		}else{
			surveyService.updateSurvey(surveyVO);
			return "redirect:/admin/survey/ViewSurvey.do?que_id="+surveyVO.getQue_id();			
		}
	}
	
	@RequestMapping(value = "DeleteSurvey.do", method = RequestMethod.GET)
	public String DeleteSurvey(HttpServletRequest request, SurveyVO surveyVO, ModelMap model) throws Exception {
		AdminVO user = (AdminVO) request.getSession().getAttribute("adminVO");
		if (user == null) {
			return "redirect:/admin/login.do";
		}else{
			surveyService.deleteSurvey(surveyVO.getQue_id());
			surveyService.deleteSurveyQueAll(surveyVO.getQue_id());
			surveyService.deleteSurveyQueItemAll(surveyVO.getQue_id());
			return "redirect:/admin/survey/survey.do?pageIndex="+surveyVO.getPageIndex()+"&que_id="+surveyVO.getQue_id();
		}
	}
	
	@RequestMapping(value = "quelist.do", method = RequestMethod.GET)
	public String quelist(HttpServletRequest request, SurveyVO surveyVO, ModelMap model) throws Exception {
		AdminVO user = (AdminVO) request.getSession().getAttribute("adminVO");
		if (user == null) {
			return "redirect:/admin/login.do";
		}else{
			SurveyVO survey = surveyService.getSurvey(surveyVO.getQue_id());
			
			List<SurveyVO> anQuestionResult = surveyService.selectAnQueList(survey);
			
			//설문조사 답변 항목 가져오기
			List<SurveyVO> surveyQueItem = surveyService.selectQueItemList(surveyVO.getQue_id());
			
			
			model.addAttribute("survey", survey);
			model.addAttribute("surveyQueItem", surveyQueItem);
			model.addAttribute("queList", anQuestionResult);
			model.addAttribute("curAdmin", user);
			
			
			return "/admin/survey/EgovSurveyAsk";
		}
	}
	
	@RequestMapping(value = "AddSurveyQueOk.do", method = RequestMethod.POST)
	public String AddSurveyQueOk(HttpServletRequest request, SurveyVO surveyVO, ModelMap model) throws Exception {
		AdminVO user = (AdminVO) request.getSession().getAttribute("adminVO");
		if (user == null) {
			return "redirect:/admin/login.do";
		}else{
			String[] queArray = surveyVO.getItem_text().split(",");
			surveyService.insertSurveyQue(surveyVO);
			
			for (int i = 0; i < queArray.length; i++) {
				surveyVO.setItem_text(queArray[i]);
				if(surveyVO.getItem_type().equals("4") && i == queArray.length-1){//기타 유형이면서 마지막 항목인 경우(주관식)
					surveyVO.setEtc_flag("Y");
				}else{
					surveyVO.setEtc_flag("");
				}
				surveyService.insertSurveyQueItem(surveyVO);
			}
			
			// 등록 후 리스트 화면이동
			return "redirect:/admin/survey/quelist.do?&que_id="+surveyVO.getQue_id();
		}
	}
	
	@RequestMapping(value = "UpdateSurveyQue.do", method = RequestMethod.GET)
	public @ResponseBody Map<String, Object> UpdateSurveyQue(HttpServletRequest request, SurveyVO surveyVO, ModelMap model) throws Exception {
		Map<String, Object> map = new HashMap<String, Object>();
		
		String anque_id = request.getParameter("anque_id")==null?"0":request.getParameter("anque_id");
		surveyVO.setAnque_id(Integer.parseInt(anque_id));
		
		
		//설문조사 질문 가져오기
		SurveyVO surveyQue = surveyService.getSurveyQue(surveyVO.getAnque_id());
		
		//설문조사 답변 항목 가져오기
		List<SurveyVO> surveyQueItem = surveyService.selectAnQueItemList(surveyVO.getAnque_id());
		
		//설문조사 답변 항목 ITEM_IDX MAX값 가져오기(수정시 답변 항목 추가, 삭제를 위한 인덱스)
		int item_idx = surveyService.getSurveyQueItemIdx(surveyVO.getAnque_id());
		
		JSONObject obj = new JSONObject();
		try {
			JSONArray jArray = new JSONArray();//배열이 필요할때
			for (int i = 0; i < surveyQueItem.size(); i++){//배열
				JSONObject sObject = new JSONObject();//배열 내에 들어갈 json
				sObject.put("itemque_id", surveyQueItem.get(i).getItemque_id()+"");
				sObject.put("que_id", surveyQueItem.get(i).getQue_id()+"");
				sObject.put("anque_id", surveyQueItem.get(i).getAnque_id()+"");
				sObject.put("item_text", surveyQueItem.get(i).getItem_text());
				sObject.put("item_idx", surveyQueItem.get(i).getItem_idx()+"");
				sObject.put("del_flag", surveyQueItem.get(i).getDel_flag());
				
				jArray.put(sObject);
			}
			
			
			JSONObject sObject = new JSONObject();//배열 내에 들어갈 json
			sObject.put("itemque_id", surveyQue.getAnque_id()+"");
			sObject.put("que_id", surveyQue.getQue_id()+"");
			sObject.put("anque_sj", surveyQue.getAnque_sj());
			sObject.put("item_type", surveyQue.getItem_type());
			sObject.put("order_num", surveyQue.getOrder_num());
			sObject.put("del_flag", surveyQue.getDel_flag());
			
			obj.put("item_idx", item_idx+"");
			obj.put("item", jArray);//배열을 넣음
			obj.put("item2", sObject);//배열을 넣음


		} catch (JSONException e) {
			e.printStackTrace();
		}
		map.put("object", obj.toString());
		return map;
	}
	
	@RequestMapping(value = "UpdateSurveyQueOk.do", method = RequestMethod.POST)
	public String UpdateSurveyQueOk(HttpServletRequest request, SurveyVO surveyVO, ModelMap model) throws Exception {
		AdminVO user = (AdminVO) request.getSession().getAttribute("adminVO");
		if (user == null) {
			return "redirect:/admin/login.do";
		}else{
			surveyService.updateSurveyQue(surveyVO);
			
			surveyService.deleteSurveyQueItemReal(surveyVO.getAnque_id());
			String[] queArray = surveyVO.getItem_text().split(",");

			for (int i = 0; i < queArray.length; i++) {
				surveyVO.setItem_text(queArray[i]);
				surveyService.insertSurveyQueItem(surveyVO);
			}
		
			return "redirect:/admin/survey/quelist.do?&que_id="+surveyVO.getQue_id();
		}
	}
	
	@RequestMapping(value = "DeleteSurveyQue.do", method = RequestMethod.GET)
	public String DeleteSurveyQue(HttpServletRequest request, SurveyVO surveyVO, ModelMap model) throws Exception {
		AdminVO user = (AdminVO) request.getSession().getAttribute("adminVO");
		if (user == null) {
			return "redirect:/admin/login.do";
		}else{
			surveyService.deleteSurveyQue(surveyVO.getAnque_id());
			surveyService.deleteSurveyQueItem(surveyVO.getAnque_id());
			
			model.addAttribute("surveyFlag", surveyVO.getSurvey_flag());
			
			return "redirect:/admin/survey/quelist.do?survey_flag="+surveyVO.getSurvey_flag()+"&que_id="+surveyVO.getQue_id();
		}
	}
	
	@RequestMapping(value = "surveyResult.do")
	public String surveyStat(SurveyVO surveyVO, HttpServletRequest request, ModelMap model) throws Exception {
		AdminVO user = (AdminVO) request.getSession().getAttribute("adminVO");
		if (user == null) {
			return "redirect:/admin/login.do";
		}else{
			SurveyVO survey = surveyService.getSurvey(surveyVO.getQue_id());
			
			List<SurveyVO> anQuestionResult = surveyService.selectAnQueList(survey);
			int anQuestionResultCnt = surveyService.selectAnQueListCnt(survey);
			//설문조사 답변 항목 가져오기
			List<SurveyVO> surveyQueItem = surveyService.selectQueItemList(surveyVO.getQue_id());
			List<SurveyVO> surveyStat = new ArrayList<SurveyVO>();
			
			for(int i=0;i<surveyQueItem.size();i++){
				SurveyVO surveyStatVO = new SurveyVO();
				surveyStatVO.setItemque_id(surveyQueItem.get(i).getItemque_id());				
				
				if(surveyQueItem.get(i).getItem_type().equals("3")){
					surveyStatVO.setSubject_answer(surveyService.getSubjectAnswer(surveyQueItem.get(i).getItemque_id()));
				}else{
					// 각 질문의 참여자 인원 가져오기
					double resultTotalCnt = (double)surveyService.getAnswerCount(surveyQueItem.get(i).getAnque_id());
					// 각 질문 항목의 답변인원 가져오기
					double resultCnt = (double)surveyService.getQuestionResultCount(surveyQueItem.get(i).getItemque_id());
					// 계산
					double resultAvg = 0;
					if(resultCnt > 0){
						resultAvg = (resultCnt / resultTotalCnt) * 100;
						resultAvg = Double.parseDouble(String.format("%.2f",resultAvg));
					}
					surveyStatVO.setAnswer_total((int)resultCnt);
					surveyStatVO.setStat_avg(resultAvg);
					
					if(surveyQueItem.get(i).getItem_type().equals("4")){
						surveyStatVO.setSubject_answer(surveyService.getSubjectAnswer(surveyQueItem.get(i).getItemque_id()));
					}
				}
				surveyStat.add(surveyStatVO);
			}
			
			String pageIndex = request.getParameter("pageIndex")==null?"1":request.getParameter("pageIndex");
			
			model.addAttribute("surveyStat", surveyStat);
			model.addAttribute("queListCnt", anQuestionResultCnt);
			model.addAttribute("queList", anQuestionResult);
			model.addAttribute("surveyQueItem", surveyQueItem);
			model.addAttribute("survey", survey);
			model.addAttribute("pageIndex", pageIndex);

			model.addAttribute("surveyVO", surveyVO);
			model.addAttribute("curAdmin", user);
			
			
			return "/admin/survey/EgovSurveyResult";
		}
	}
	
	@RequestMapping(value = "ExcelDown.do")
	public ModelAndView excelDown(SurveyVO surveyVO, HttpServletRequest request, ModelMap model) throws Exception {
		
		int que_id = Integer.parseInt(request.getParameter("que_id"));
		SurveyVO survey = surveyService.getSurvey(que_id);
		
		List<SurveyVO> anQuestionResult = surveyService.selectAnQueList(survey);
		//설문조사 답변 항목 가져오기
		List<SurveyVO> surveyQueItem = surveyService.selectQueItemList(que_id);
		List<SurveyVO> surveyStat = new ArrayList<SurveyVO>();
		
		for(int i=0;i<surveyQueItem.size();i++){
			SurveyVO surveyStatVO = new SurveyVO();
			surveyStatVO.setItemque_id(surveyQueItem.get(i).getItemque_id());
			
			if(surveyQueItem.get(i).getItem_type().equals("3")){
				surveyStatVO.setSubject_answer(surveyService.getSubjectAnswer(surveyQueItem.get(i).getItemque_id()));
			}else{
				// 각 질문의 참여자 인원 가져오기
				double resultTotalCnt = (double)surveyService.getAnswerCount(surveyQueItem.get(i).getAnque_id());
				// 각 질문 항목의 답변인원 가져오기
				double resultCnt = (double)surveyService.getQuestionResultCount(surveyQueItem.get(i).getItemque_id());
				// 계산
				double resultAvg = 0;
				if(resultCnt > 0){
					resultAvg = (resultCnt / resultTotalCnt) * 100;
					resultAvg = Double.parseDouble(String.format("%.2f",resultAvg));
				}
				surveyStatVO.setAnswer_total((int)resultCnt);
				surveyStatVO.setStat_avg(resultAvg);
				
				if(surveyQueItem.get(i).getItem_type().equals("4")){
					surveyStatVO.setSubject_answer(surveyService.getSubjectAnswer(surveyQueItem.get(i).getItemque_id()));
				}
			}
			surveyStat.add(surveyStatVO);
		}
		
		String excelFlag = (request.getParameter("excelFlag") == null ? "" : request.getParameter("excelFlag"));
		BoardVO boardVO = new BoardVO();
		boardVO.setExcel_flag(excelFlag);
		
		model.addAttribute("boardVO", boardVO);
		model.addAttribute("excelFlag", excelFlag);
		model.addAttribute("surveyStat", surveyStat);
		model.addAttribute("queList", anQuestionResult);
		model.addAttribute("surveyQueItem", surveyQueItem);
		model.addAttribute("survey", survey);
		model.addAttribute("queId", que_id);
		model.addAttribute("surveyVO", surveyVO);
		
		// 현재 날짜 구하기
		SimpleDateFormat mSimpleDateFormat = new SimpleDateFormat("yyyy-MM-dd",Locale.KOREA);
		Date currentTime = new Date();
		String mTime = mSimpleDateFormat.format(currentTime);
		model.addAttribute("currentDate", mTime);
		
		return new ModelAndView("excelDownView", "categoryMap", model);
	}
}

